@@ -19,15 +19,15 @@ extern const GUI::gui_char propAbbrevFileName[];
 #endif
 #endif
 
-inline int Minimum(int a, int b) {
+inline int Minimum(int a, int b) noexcept {
 	return (a < b) ? a : b;
 }
 
-inline int Maximum(int a, int b) {
+inline int Maximum(int a, int b) noexcept {
 	return (a > b) ? a : b;
 }
 
-inline long LongFromTwoShorts(short a,short b) {
+inline long LongFromTwoShorts(short a,short b) noexcept {
 	return (a) | ((b) << 16);
 }
 
@@ -44,7 +44,7 @@ enum {
 struct SelectedRange {
 	int position;
 	int anchor;
-	SelectedRange(int position_= INVALID_POSITION, int anchor_= INVALID_POSITION) :
+	SelectedRange(int position_= INVALID_POSITION, int anchor_= INVALID_POSITION) noexcept :
 		position(position_), anchor(anchor_) {
 	}
 };
@@ -110,7 +110,7 @@ public:
 	Buffer() :
 			file(), doc(0), isDirty(false), isReadOnly(false), failedSave(false), useMonoFont(false), lifeState(empty),
 			unicodeMode(uni8Bit), fileModTime(0), fileModLastAsk(0), documentModTime(0),
-			findMarks(fmNone), pFileWorker(0), futureDo(fdNone) {}
+			findMarks(fmNone), pFileWorker(nullptr), futureDo(fdNone) {}
 
 	~Buffer() = default;
 	void Init() {
@@ -128,7 +128,7 @@ public:
 		overrideExtension = "";
 		foldState.clear();
 		bookmarks.clear();
-		pFileWorker = 0;
+		pFileWorker = nullptr;
 		futureDo = fdNone;
 	}
 
@@ -146,7 +146,7 @@ public:
 	void CompleteStoring();
 	void AbandonAutomaticSave();
 
-	bool ShouldNotSave() const {
+	bool ShouldNotSave() const noexcept {
 		return lifeState != open;
 	}
 
@@ -174,7 +174,7 @@ public:
 
 	BufferList();
 	~BufferList();
-	int size() const {
+	int size() const noexcept {
 		return static_cast<int>(buffers.size());
 	}
 	void Allocate(int maxSize);
@@ -183,10 +183,10 @@ public:
 	int GetDocumentByName(const FilePath &filename, bool excludeCurrent=false);
 	void RemoveInvisible(int index);
 	void RemoveCurrent();
-	int Current() const;
+	int Current() const noexcept;
 	Buffer *CurrentBuffer();
 	const Buffer *CurrentBufferConst() const;
-	void SetCurrent(int index);
+	void SetCurrent(int index) noexcept;
 	int StackNext();
 	int StackPrev();
 	void CommitStackSelection();
@@ -196,7 +196,7 @@ public:
 	bool SingleBuffer() const;
 	BackgroundActivities CountBackgroundActivities() const;
 	bool SavingInBackground() const;
-	bool GetVisible(int index) const;
+	bool GetVisible(int index) const noexcept;
 	void SetVisible(int index, bool visible);
 	void AddFuture(int index, Buffer::FutureDo fd);
 	void FinishedFuture(int index, Buffer::FutureDo fd);
@@ -247,10 +247,10 @@ enum IndentationStatus {
 struct StyleAndWords {
 	int styleNumber;
 	std::string words;
-	StyleAndWords() : styleNumber(0) {
+	StyleAndWords() noexcept : styleNumber(0) {
 	}
-	bool IsEmpty() const { return words.length() == 0; }
-	bool IsSingleChar() const { return words.length() == 1; }
+	bool IsEmpty() const noexcept { return words.length() == 0; }
+	bool IsSingleChar() const noexcept { return words.length() == 1; }
 };
 
 struct CurrentWordHighlight {
@@ -268,8 +268,8 @@ struct CurrentWordHighlight {
 	CurrentWordHighlight() {
 		statesOfDelay = noDelay;
 		isEnabled = false;
-		isOnlyWithSameStyle = false;
 		textHasChanged = false;
+		isOnlyWithSameStyle = false;
 	}
 };
 
@@ -329,8 +329,8 @@ public:
 	virtual void ReplaceOnce(bool showWarnings=true) = 0;
 	virtual void UIClosed() = 0;
 	virtual void UIHasFocus() = 0;
-	bool &FlagFromCmd(int cmd);
-	bool ShouldClose(bool found) const {
+	bool &FlagFromCmd(int cmd) noexcept;
+	bool ShouldClose(bool found) const noexcept {
 		return (closeFind == CloseFind::closeAlways) || (found && (closeFind == CloseFind::closeOnMatch));
 	}
 };
@@ -347,9 +347,9 @@ class SearchUI {
 protected:
 	Searcher *pSearcher;
 public:
-	SearchUI() : pSearcher(0) {
+	SearchUI() noexcept : pSearcher(nullptr) {
 	}
-	void SetSearcher(Searcher *pSearcher_) {
+	void SetSearcher(Searcher *pSearcher_) noexcept {
 		pSearcher = pSearcher_;
 	}
 };
@@ -580,7 +580,7 @@ protected:
 	void ReadLocalPropFile();
 	void ReadDirectoryPropFile();
 
-	void SetPaneFocus(bool editPane);
+	void SetPaneFocus(bool editPane) noexcept;
 	int CallFocused(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	int CallFocusedElseDefault(int defaultValue, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	sptr_t CallPane(int destination, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
@@ -734,7 +734,7 @@ protected:
 	};
 	virtual MessageBoxChoice WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, MessageBoxStyle style = mbsIconWarning) = 0;
 	void FailedSaveMessageBox(const FilePath &filePathSaving);
-	virtual void FindMessageBox(const std::string &msg, const std::string *findItem = 0) = 0;
+	virtual void FindMessageBox(const std::string &msg, const std::string *findItem = nullptr) = 0;
 	bool FindReplaceAdvanced() const;
 	int FindInTarget(const std::string &findWhatText, int startPosition, int endPosition);
 	// Implement Searcher
@@ -853,7 +853,7 @@ protected:
 	virtual void SizeSubWindows() = 0;
 
 	virtual void SetMenuItem(int menuNumber, int position, int itemID,
-		const GUI::gui_char *text, const GUI::gui_char *mnemonic = 0) = 0;
+		const GUI::gui_char *text, const GUI::gui_char *mnemonic = nullptr) = 0;
 	virtual void RedrawMenu() {}
 	virtual void DestroyMenuItem(int menuNumber, int itemID) = 0;
 	virtual void CheckAMenuItem(int wIDCheckItem, bool val) = 0;
@@ -862,6 +862,7 @@ protected:
 	virtual void CheckMenus();
 	virtual void AddToPopUp(const char *label, int cmd = 0, bool enabled = true) = 0;
 	void ContextMenu(GUI::ScintillaWindow &wSource, GUI::Point pt, GUI::Window wCmd);
+	static std::string MenuPathToMenuKeyString(const char *path);
 
 	void DeleteFileStackMenu();
 	void SetFileStackMenu();
@@ -890,7 +891,7 @@ protected:
 	void SetLanguageMenu();
 	void SetPropertiesInitial();
 	GUI::gui_string LocaliseMessage(const char *s,
-		const GUI::gui_char *param0 = 0, const GUI::gui_char *param1 = 0, const GUI::gui_char *param2 = 0);
+		const GUI::gui_char *param0 = nullptr, const GUI::gui_char *param1 = nullptr, const GUI::gui_char *param2 = nullptr);
 	virtual void ReadLocalization();
 	std::string GetFileNameProperty(const char *name);
 	virtual void ReadPropertiesInitial();
@@ -989,7 +990,7 @@ public:
 
 	void Finalise();
 
-	GUI::WindowID GetID() const { return wSciTE.GetID(); }
+	GUI::WindowID GetID() const noexcept { return wSciTE.GetID(); }
 
 	virtual bool PerformOnNewThread(Worker *pWorker) = 0;
 	// WorkerListener
@@ -997,10 +998,10 @@ public:
 	virtual void WorkerCommand(int cmd, Worker *pWorker);
 };
 
-int ControlIDOfCommand(unsigned long);
+int ControlIDOfCommand(unsigned long) noexcept;
 long ColourOfProperty(const PropSetFile &props, const char *key, Colour colourDefault);
 void WindowSetFocus(GUI::ScintillaWindow &w);
 
-inline bool isspacechar(unsigned char ch) {
+inline bool isspacechar(unsigned char ch) noexcept {
     return (ch == ' ') || ((ch >= 0x09) && (ch <= 0x0d));
 }
