@@ -46,11 +46,11 @@
 const GUI::gui_char defaultSessionFileName[] = GUI_TEXT("SciTE.session");
 
 void Buffer::DocumentModified() {
-	documentModTime = time(0);
+	documentModTime = time(nullptr);
 }
 
 bool Buffer::NeedsSave(int delayBeforeSave) const {
-	const time_t now = time(0);
+	const time_t now = time(nullptr);
 	return now && documentModTime && isDirty && !pFileWorker && (now-documentModTime > delayBeforeSave) && !file.IsUntitled() && !failedSave;
 }
 
@@ -58,14 +58,14 @@ void Buffer::CompleteLoading() {
 	lifeState = open;
 	if (pFileWorker && pFileWorker->IsLoading()) {
 		delete pFileWorker;
-		pFileWorker = 0;
+		pFileWorker = nullptr;
 	}
 }
 
 void Buffer::CompleteStoring() {
 	if (pFileWorker && !pFileWorker->IsLoading()) {
 		delete pFileWorker;
-		pFileWorker = 0;
+		pFileWorker = nullptr;
 	}
 	SetTimeFromFile();
 }
@@ -178,7 +178,7 @@ void BufferList::RemoveCurrent() {
 	MoveToStackTop(current);
 }
 
-int BufferList::Current() const {
+int BufferList::Current() const noexcept {
 	return current;
 }
 
@@ -190,7 +190,7 @@ const Buffer *BufferList::CurrentBufferConst() const {
 	return &buffers[Current()];
 }
 
-void BufferList::SetCurrent(int index) {
+void BufferList::SetCurrent(int index) noexcept {
 	current = index;
 }
 
@@ -317,7 +317,7 @@ bool BufferList::SavingInBackground() const {
 	return false;
 }
 
-bool BufferList::GetVisible(int index) const {
+bool BufferList::GetVisible(int index) const noexcept {
 	return index < lengthVisible;
 }
 
@@ -541,7 +541,7 @@ static std::string IndexPropKey(const char *bufPrefix, int bufIndex, const char
 	std::string pKey = bufPrefix;
 	pKey += '.';
 	pKey += StdStringFromInteger(bufIndex + 1);
-	if (bufAppendix != NULL) {
+	if (bufAppendix) {
 		pKey += ".";
 		pKey += bufAppendix;
 	}
@@ -1441,7 +1441,7 @@ void SciTEBase::SetToolsMenu() {
 				sMnemonic += StdStringFromInteger(item);
 			}
 			SetMenuItemLocalised(menuTools, menuPos, itemID, sMenuItem.c_str(),
-				sMnemonic.length() ? sMnemonic.c_str() : NULL);
+				sMnemonic.length() ? sMnemonic.c_str() : nullptr);
 			menuPos++;
 		}
 	}
@@ -1497,7 +1497,7 @@ void SciTEBase::ToolsMenu(int item) {
 	}
 }
 
-inline bool isdigitchar(int ch) {
+inline bool isdigitchar(int ch) noexcept {
 	return (ch >= '0') && (ch <= '9');
 }
 
@@ -1586,7 +1586,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 					space++;
 				}
 
-				const char *space2 = NULL;
+				const char *space2 = nullptr;
 
 				if (strlen(space) > 2) {
 					space2 = strchr(space + 2, ':');
@@ -1615,11 +1615,13 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			// perl
 			const char *at = strstr(cdoc, " at ");
 			const char *line = strstr(cdoc, " line ");
-			const ptrdiff_t length = line - (at + 4);
-			if (at && line && length > 0) {
-				sourcePath.assign(at + 4, length);
-				line += 6;
-				return atoi(line) - 1;
+			if (at && line) {
+				const ptrdiff_t length = line - (at + 4);
+				if (length > 0) {
+					sourcePath.assign(at + 4, length);
+					line += 6;
+					return atoi(line) - 1;
+				}
 			}
 			break;
 		}
@@ -1646,9 +1648,11 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			if (line && file) {
 				const char *fileStart = file + lenFile + 1;
 				const char *quote = strstr(fileStart, "'");
-				const size_t length = quote - fileStart;
-				if (quote && length > 0) {
-					sourcePath.assign(fileStart, length);
+				if (quote) {
+					const size_t length = quote - fileStart;
+					if (length > 0) {
+						sourcePath.assign(fileStart, length);
+					}
 				}
 				line += lenLine;
 				return atoi(line) - 1;
@@ -1846,7 +1850,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 
 #define CSI "\033["
 
-static bool SeqEnd(int ch) {
+static bool SeqEnd(int ch) noexcept {
 	return (ch == 0) || ((ch >= '@') && (ch <= '~'));
 }
 
@@ -1906,7 +1910,7 @@ void SciTEBase::ShowMessages(int line) {
 					}
 				}
 			}
-			const int lenCurrent = wEditor.CallString(SCI_ANNOTATIONGETTEXT, sourceLine, NULL);
+			const int lenCurrent = wEditor.CallString(SCI_ANNOTATIONGETTEXT, sourceLine, nullptr);
 			std::string msgCurrent(lenCurrent, '\0');
 			std::string stylesCurrent(lenCurrent, '\0');
 			if (lenCurrent) {
@@ -1985,7 +1989,7 @@ void SciTEBase::GoMessage(int dir) {
 						bExists = true;
 					} else if (Exists(filePath.Directory().AsInternal(), sourceString.c_str(), &messagePath)) {
 						bExists = true;
-					} else if (Exists(NULL, sourceString.c_str(), &messagePath)) {
+					} else if (Exists(nullptr, sourceString.c_str(), &messagePath)) {
 						bExists = true;
 					} else {
 						// Look through buffers for name match
