@@ -275,7 +275,7 @@ void SciTEBase::OpenCurrentFile(long long fileSize, bool suppressMessage, bool a
 		// Turn grey while loading
 		wEditor.Call(SCI_STYLESETBACK, STYLE_DEFAULT, 0xEEEEEE);
 		wEditor.Call(SCI_SETREADONLY, 1);
-		assert(CurrentBufferConst()->pFileWorker == NULL);
+		assert(CurrentBufferConst()->pFileWorker == nullptr);
 		ILoader *pdocLoad;
 		try {
 			sptr_t docOptions = SC_DOCUMENTOPTION_DEFAULT;
@@ -312,7 +312,7 @@ void SciTEBase::OpenCurrentFile(long long fileSize, bool suppressMessage, bool a
 			lenFile = fread(&data[0], 1, data.size(), fp);
 			if (lenFile == 0) {
 				// Handle case where convert is holding a lead surrogate but no more data
-				const size_t lenFileTrail = convert.convert(NULL, lenFile);
+				const size_t lenFileTrail = convert.convert(nullptr, lenFile);
 				if (lenFileTrail) {
 					const char *dataTrail = convert.getNewBuf();
 					wEditor.CallString(SCI_ADDTEXT, lenFileTrail, dataTrail);
@@ -348,7 +348,7 @@ void SciTEBase::TextRead(FileWorker *pFileWorker) {
 		}
 		// Switch documents
 		const sptr_t pdocLoading = reinterpret_cast<sptr_t>(pFileLoader->pLoader->ConvertToDocument());
-		pFileLoader->pLoader = 0;
+		pFileLoader->pLoader = nullptr;
 		SwitchDocumentAt(iBuffer, pdocLoading);
 		if (iBuffer == buffers.Current()) {
 			CompleteOpen(ocCompleteCurrent);
@@ -573,7 +573,7 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 		}
 	}
 
-	assert(CurrentBufferConst()->pFileWorker == NULL);
+	assert(CurrentBufferConst()->pFileWorker == nullptr);
 	SetFileName(absPath);
 
 	propsDiscovered.Clear();
@@ -1208,7 +1208,7 @@ bool SciTEBase::Save(SaveFlags sf) {
 		return true;
 	} else {
 		if (props.GetString("save.path.suggestion").length()) {
-			const time_t t = time(NULL);
+			const time_t t = time(nullptr);
 			char timeBuff[15];
 			strftime(timeBuff, sizeof(timeBuff), "%Y%m%d%H%M%S",  localtime(&t));
 			PropSetFile propsSuggestion;
@@ -1338,7 +1338,7 @@ void SciTEBase::OpenFilesFromStdin() {
 
 	while (fgets(data, sizeof(data) - 1, stdin)) {
 		char *pNL;
-		if ((pNL = strchr(data, '\n')) != NULL)
+		if ((pNL = strchr(data, '\n')) != nullptr)
 			* pNL = '\0';
 		Open(GUI::StringFromUTF8(data).c_str(), ofQuiet);
 	}
@@ -1371,7 +1371,7 @@ public:
 	explicit BufferedFile(const FilePath &fPath) {
 		fp = fPath.Open(fileRead);
 		readAll = false;
-		exhausted = fp == NULL;
+		exhausted = fp == nullptr;
 		buffer[0] = 0;
 		pos = 0;
 		valid = 0;
@@ -1380,9 +1380,9 @@ public:
 		if (fp) {
 			fclose(fp);
 		}
-		fp = NULL;
+		fp = nullptr;
 	}
-	bool Exhausted() const {
+	bool Exhausted() const noexcept {
 		return exhausted;
 	}
 	int NextByte() {
@@ -1421,7 +1421,7 @@ public:
 	}
 	const char *Next() {
 		if (bf->Exhausted()) {
-			return NULL;
+			return nullptr;
 		}
 		lineToShow.clear();
 		while (!bf->Exhausted()) {
@@ -1442,10 +1442,10 @@ public:
 		}
 		return lineToCompare.c_str();
 	}
-	int LineNumber() const {
+	int LineNumber() const noexcept {
 		return lineNum;
 	}
-	const char *Original() const {
+	const char *Original() const noexcept {
 		return lineToShow.c_str();
 	}
 	bool BufferContainsNull() {
@@ -1453,7 +1453,7 @@ public:
 	}
 };
 
-static bool IsWordCharacter(int ch) {
+static bool IsWordCharacter(int ch) noexcept {
 	return (ch >= 'A' && ch <= 'Z') || (ch >= 'a' && ch <= 'z')  || (ch >= '0' && ch <= '9')  || (ch == '_');
 }
 
