@@ -430,7 +430,7 @@ int ViewStyle::ExternalMarginWidth() const {
 
 int ViewStyle::MarginFromLocation(Point pt) const {
 	int margin = -1;
-	int x = textStart - fixedColumnWidth;
+	int x = marginInside ? 0 : -fixedColumnWidth;
 	for (size_t i = 0; i < ms.size(); i++) {
 		if ((pt.x >= x) && (pt.x < x + ms[i].width))
 			margin = static_cast<int>(i);
