@@ -51,7 +51,7 @@ static bool HasConnection() {
 	return (wDirector != 0) || (wCorrespondent != 0);
 }
 
-static void SendDirector(const char *verb, const char *arg = 0) {
+static void SendDirector(const char *verb, const char *arg = nullptr) {
 	if (HasConnection()) {
 		HWND wDestination = wCorrespondent;
 		std::string addressedMessage;
@@ -72,8 +72,7 @@ static void SendDirector(const char *verb, const char *arg = 0) {
 		cds.dwData = 0;
 		cds.cbData = static_cast<int>(slashedMessage.length());
 		slashedMessage.append(1, '\0');	// Ensure NUL at end of string
-		cds.lpData = static_cast<void *>(
-			                &slashedMessage[0]);
+		cds.lpData = &slashedMessage[0];
 		::SendMessage(wDestination, WM_COPYDATA,
 			            reinterpret_cast<WPARAM>(wReceiver),
 			            reinterpret_cast<LPARAM>(&cds));
@@ -137,7 +136,7 @@ static void DirectorExtension_Register(HINSTANCE hInstance) {
 	wndclass.hIcon = 0;
 	wndclass.hCursor = NULL;
 	wndclass.hbrBackground = NULL;
-	wndclass.lpszMenuName = 0;
+	wndclass.lpszMenuName = nullptr;
 	wndclass.lpszClassName = DirectorExtension_ClassName;
 	if (!::RegisterClass(&wndclass))
 		::exit(FALSE);
