@@ -117,6 +117,10 @@ long SciTEKeys::ParseKeyCode(const char *mnemonic) {
 				keyval = VK_ESCAPE;
 			} else if (sKey == "Delete") {
 				keyval = VK_DELETE;
+			} else if (sKey == "Del") {
+				keyval = VK_DELETE;
+			} else if (sKey == "Break") {
+				keyval = VK_CANCEL;
 			} else if (sKey == "PageUp") {
 				keyval = VK_PRIOR;
 			} else if (sKey == "PageDown") {
@@ -132,7 +136,7 @@ long SciTEKeys::ParseKeyCode(const char *mnemonic) {
 	return (keyval > 0) ? (keyval | (modsInKey<<16)) : 0;
 }
 
-bool SciTEKeys::MatchKeyCode(long parsedKeyCode, int keyval, int modifiers) {
+bool SciTEKeys::MatchKeyCode(long parsedKeyCode, int keyval, int modifiers) noexcept {
 	return parsedKeyCode && !(0xFFFF0000 & (keyval | modifiers)) && (parsedKeyCode == (keyval | (modifiers<<16)));
 }
 
@@ -435,7 +439,7 @@ void SciTEWin::ReadEmbeddedProperties() {
 void SciTEWin::ReadPropertiesInitial() {
 	SciTEBase::ReadPropertiesInitial();
 	if (tabMultiLine) {	// Windows specific!
-		const long wl = ::GetWindowLong(HwndOf(wTabBar), GWL_STYLE);
+		const long wl = GetWindowStyle(HwndOf(wTabBar));
 		::SetWindowLong(HwndOf(wTabBar), GWL_STYLE, wl | TCS_MULTILINE);
 	}
 }
@@ -553,18 +557,17 @@ void SciTEWin::CopyAsRTF() {
 	const Sci_CharacterRange cr = GetSelection();
 	std::ostringstream oss;
 	SaveToStreamRTF(oss, static_cast<int>(cr.cpMin), static_cast<int>(cr.cpMax));
-	std::string rtf = oss.str();
-	size_t len = rtf.length();
-	HGLOBAL hand = ::GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, len + 1);
+	const std::string rtf = oss.str();
+	const size_t len = rtf.length() + 1;	// +1 for NUL
+	HGLOBAL hand = ::GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, len);
 	if (hand) {
 		::OpenClipboard(MainHWND());
 		::EmptyClipboard();
 		char *ptr = static_cast<char *>(::GlobalLock(hand));
 		if (ptr) {
 			memcpy(ptr, rtf.c_str(), len);
-			ptr[len] = '\0';
+			::GlobalUnlock(hand);
 		}
-		::GlobalUnlock(hand);
 		::SetClipboardData(::RegisterClipboardFormat(CF_RTF), hand);
 		::CloseClipboard();
 	}
@@ -574,16 +577,17 @@ void SciTEWin::CopyPath() {
 	if (filePath.IsUntitled())
 		return;
 
-	GUI::gui_string clipText(filePath.AsInternal());
+	const GUI::gui_string clipText(filePath.AsInternal());
 	const size_t blobSize = sizeof(GUI::gui_char)*(clipText.length()+1);
 	if (::OpenClipboard(MainHWND())) {
 		HGLOBAL hand = ::GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, blobSize);
 		if (hand) {
 			::EmptyClipboard();
 			GUI::gui_char *ptr = static_cast<GUI::gui_char*>(::GlobalLock(hand));
-			if (ptr)
+			if (ptr) {
 				memcpy(ptr, clipText.c_str(), blobSize);
-			::GlobalUnlock(hand);
+				::GlobalUnlock(hand);
+			}
 			::SetClipboardData(CF_UNICODETEXT, hand);
 		}
 		::CloseClipboard();
@@ -1297,7 +1301,8 @@ static void WorkerThread(void *ptr) {
 }
 
 bool SciTEWin::PerformOnNewThread(Worker *pWorker) {
-	const uintptr_t result = _beginthread(WorkerThread, 1024 * 1024, static_cast<void *>(pWorker));
+	void *threadArgument = pWorker;
+	const uintptr_t result = _beginthread(WorkerThread, 1024 * 1024, threadArgument);
 	return result != static_cast<uintptr_t>(-1);
 }
 
@@ -1403,7 +1408,7 @@ void SciTEWin::CreateUI() {
 	UIAvailable();
 }
 
-static bool IsASpace(int ch) {
+static bool IsASpace(int ch) noexcept {
 	return (ch == ' ') || (ch == '\t');
 }
 
@@ -1544,7 +1549,7 @@ void ContentWin::Paint(HDC hDC, GUI::Rectangle) {
 				colourIndex = COLOR_3DFACE;
 		}
 		HPEN pen = ::CreatePen(0, 1, ::GetSysColor(colourIndex));
-		HPEN penOld = static_cast<HPEN>(::SelectObject(hDC, pen));
+		HPEN penOld = SelectPen(hDC, pen);
 		if (pSciTEWin->splitVertical) {
 			::MoveToEx(hDC, xBorder + i, 0, 0);
 			::LineTo(hDC, xBorder + i, heightClient);
@@ -1552,8 +1557,8 @@ void ContentWin::Paint(HDC hDC, GUI::Rectangle) {
 			::MoveToEx(hDC, 0, yBorder + i, 0);
 			::LineTo(hDC, widthClient, yBorder + i);
 		}
-		::SelectObject(hDC, penOld);
-		::DeleteObject(pen);
+		SelectPen(hDC, penOld);
+		DeletePen(pen);
 	}
 }
 
@@ -1811,6 +1816,17 @@ LRESULT SciTEWin::KeyDown(WPARAM wParam) {
 			}
 		}
 	}
+	
+	// like MatchKeyCode, confirm high bits aren't set before combining key and mods into one int
+	const int key = static_cast<int>(wParam);
+	if (!(0xFFFF0000 & (key | modifiers))) {
+		const int combined = key | (modifiers<<16);
+		std::map<long, int>::const_iterator it = currentMenuKeys.find(combined);
+		if (it != currentMenuKeys.end()) {
+			SciTEBase::MenuCommand(it->second);
+			return 1l;
+		}
+	}
 
 	return 0;
 }
@@ -2052,21 +2068,18 @@ LRESULT SciTEWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 
 LRESULT PASCAL SciTEWin::TWndProc(
     HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam) {
-
+	if (iMessage == WM_CREATE) {
+		SciTEWin *scitePassed = static_cast<SciTEWin *>(SetWindowPointerFromCreate(hWnd, lParam));
+		scitePassed->wSciTE = hWnd;
+	}
 	// Find C++ object associated with window.
 	SciTEWin *scite = static_cast<SciTEWin *>(PointerFromWindow(hWnd));
 	// scite will be zero if WM_CREATE not seen yet
-	if (!scite) {
-		if (iMessage == WM_CREATE) {
-			LPCREATESTRUCT cs = reinterpret_cast<LPCREATESTRUCT>(lParam);
-			scite = static_cast<SciTEWin *>(cs->lpCreateParams);
-			scite->wSciTE = hWnd;
-			SetWindowPointer(hWnd, scite);
-			return scite->WndProc(iMessage, wParam, lParam);
-		} else
-			return ::DefWindowProcW(hWnd, iMessage, wParam, lParam);
-	} else
+	if (scite) {
 		return scite->WndProc(iMessage, wParam, lParam);
+	} else {
+		return ::DefWindowProcW(hWnd, iMessage, wParam, lParam);
+	}
 }
 
 LRESULT ContentWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
