@@ -370,7 +370,6 @@ void SciTEWin::SizeSubWindows() {
 	const GUI::Rectangle rcClient = wSciTE.GetClientPosition();
 	bool showTab = false;
 
-	//::SendMessage(MainHWND(), WM_SETREDRAW, false, 0); // suppress flashing
 	visHeightTools = tbVisible ? (tbLarge ? heightToolsBig : heightTools) : 0;
 	bands[bandTool].visible = tbVisible;
 
@@ -449,8 +448,6 @@ void SciTEWin::SizeSubWindows() {
 	visHeightStatus = bands[bandStatus].height;
 
 	SizeContentWindows();
-	//::SendMessage(MainHWND(), WM_SETREDRAW, true, 0);
-	//::RedrawWindow(MainHWND(), NULL, NULL, RDW_INVALIDATE | RDW_ALLCHILDREN);
 }
 
 
@@ -542,7 +539,83 @@ void SciTEWin::CheckMenus() {
 	                   CurrentBuffer()->unicodeMode + IDM_ENCODING_DEFAULT, 0);
 }
 
-void SciTEWin::LocaliseMenu(HMENU hmenu) {
+namespace {
+
+void MenuKeyStringGtkToWindows(std::string &keys) {
+	Substitute(keys, "<control>", "Ctrl+");
+	Substitute(keys, "<shift>", "Shift+");
+	Substitute(keys, "<alt>", "Alt+");
+}
+
+bool AllowRedefineMenuKeys(const std::string &keys) {
+	// We don't allow these to be redefined by the user, since we want them to be 
+	// functional for other text fields, like the text box in the find strip
+	return keys != "Ctrl+C" &&
+		keys != "Ctrl+X" &&
+		keys != "Ctrl+V" &&
+		keys != "Ctrl+A" &&
+		keys != "Ctrl+Z" &&
+		keys != "Ctrl+Y";
+}
+
+}
+
+std::string SciTEWin::GetMenuKeys(const GUI::gui_string &path, const GUI::gui_string &defaultAccel) const {
+	// Did the user specify a different accelerator key for this menuitem?
+	std::string menuKeyProp = MenuPathToMenuKeyString(GUI::UTF8FromString(path.c_str()).c_str());
+	std::string fromProps = props.GetString(menuKeyProp.c_str());
+	if (fromProps.length() > 0) {
+		// Allow user to clear accelerator key
+		if (fromProps == "none" || fromProps == "\"\"") {
+			fromProps = "";
+		}
+		
+		MenuKeyStringGtkToWindows(fromProps);
+		return AllowRedefineMenuKeys(fromProps) ? fromProps : "";
+	} else {
+		// Fall back to default accelerator key
+		return GUI::UTF8FromString(defaultAccel);
+	}
+}
+
+void SciTEWin::LocaliseMenuItem(HMENU hmenu, int i, MENUITEMINFOW &mii, const GUI::gui_string &path) {
+	// Get the default text and default accelerator key
+	GUI::gui_string text;
+	GUI::gui_string defaultAccel;
+	const GUI::gui_string textAndAccel(mii.dwTypeData);
+	const std::vector<GUI::gui_string> spl = StringSplit(textAndAccel, GUI_TEXT('\t'));
+	if (spl.size() > 1) {
+		text = spl[0];
+		defaultAccel = spl[1];
+	} else if (spl.size() > 0) {
+		text = spl[0];
+	}
+	
+	// Register in currentMenuKeys
+	const GUI::gui_string fullPath(path + GUI_TEXT("/") + text);
+	const std::string accel = GetMenuKeys(fullPath, defaultAccel);
+	if (accel.length() > 0 && AllowRedefineMenuKeys(accel)) {
+		const long parsedKeys = SciTEKeys::ParseKeyCode(accel.c_str());
+		if (parsedKeys) {
+			const int action = ::GetMenuItemID(hmenu, i);
+			currentMenuKeys[parsedKeys] = action;
+		}
+	}
+	
+	// Localise text
+	text = localiser.Text(GUI::UTF8FromString(text.c_str()).c_str(), true);
+	if (text.length()) {
+		if (accel != "") {
+			text += GUI_TEXT("\t");
+			text += GUI::StringFromUTF8(accel);
+		}
+		text.append(1, 0);
+		mii.dwTypeData = &text[0];
+		::SetMenuItemInfoW(hmenu, i, TRUE, &mii);
+	}
+}
+
+void SciTEWin::LocaliseMenu(HMENU hmenu, GUI::gui_string path) {
 	for (int i = 0; i <= ::GetMenuItemCount(hmenu); i++) {
 		GUI::gui_char buff[200];
 		buff[0] = '\0';
@@ -554,30 +627,11 @@ void SciTEWin::LocaliseMenu(HMENU hmenu) {
 		mii.cch = sizeof(buff) - 1;
 		if (::GetMenuItemInfoW(hmenu, i, TRUE, &mii)) {
 			if (mii.hSubMenu) {
-				LocaliseMenu(mii.hSubMenu);
+				LocaliseMenu(mii.hSubMenu, path + GUI_TEXT("/") + mii.dwTypeData);
 			}
 			if (mii.fType == MFT_STRING || mii.fType == MFT_RADIOCHECK) {
 				if (mii.dwTypeData) {
-					GUI::gui_string text(mii.dwTypeData);
-					GUI::gui_string accel(mii.dwTypeData);
-					const size_t len = text.length();
-					const size_t tab = text.find(GUI_TEXT("\t"));
-					if (tab != GUI::gui_string::npos) {
-						text.erase(tab, len - tab);
-						accel.erase(0, tab + 1);
-					} else {
-						accel = GUI_TEXT("");
-					}
-					text = localiser.Text(GUI::UTF8FromString(text.c_str()).c_str(), true);
-					if (text.length()) {
-						if (accel != GUI_TEXT("")) {
-							text += GUI_TEXT("\t");
-							text += accel;
-						}
-						text.append(1, 0);
-						mii.dwTypeData = &text[0];
-						::SetMenuItemInfoW(hmenu, i, TRUE, &mii);
-					}
+					LocaliseMenuItem(hmenu, i, mii, path);
 				}
 			}
 		}
@@ -585,7 +639,7 @@ void SciTEWin::LocaliseMenu(HMENU hmenu) {
 }
 
 void SciTEWin::LocaliseMenus() {
-	LocaliseMenu(::GetMenu(MainHWND()));
+	LocaliseMenu(::GetMenu(MainHWND()), GUI_TEXT(""));
 	::DrawMenuBar(MainHWND());
 }
 
@@ -598,10 +652,10 @@ void SciTEWin::LocaliseControl(HWND w) {
 
 void SciTEWin::LocaliseDialog(HWND wDialog) {
 	LocaliseControl(wDialog);
-	HWND wChild = ::GetWindow(wDialog, GW_CHILD);
+	HWND wChild = GetFirstChild(wDialog);
 	while (wChild) {
 		LocaliseControl(wChild);
-		wChild = ::GetWindow(wChild, GW_HWNDNEXT);
+		wChild = GetNextSibling(wChild);
 	}
 }
 
@@ -781,15 +835,15 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 						};
 
 						HPEN pen = ::CreatePen(0,1,RGB(255, 0, 0));
-						HPEN penOld = static_cast<HPEN>(::SelectObject(hDC, pen));
+						HPEN penOld = SelectPen(hDC, pen);
 						const COLORREF colourNearest = ::GetNearestColor(hDC, RGB(255, 0, 0));
 						HBRUSH brush = ::CreateSolidBrush(colourNearest);
-						HBRUSH brushOld = static_cast<HBRUSH>(::SelectObject(hDC, brush));
+						HBRUSH brushOld = SelectBrush(hDC, brush);
 						::Polygon(hDC, tab < st_iDraggingTab ? ptsLeftArrow : ptsRightArrow, 7);
-						::SelectObject(hDC, brushOld);
-						::DeleteObject(brush);
-						::SelectObject(hDC, penOld);
-						::DeleteObject(pen);
+						SelectBrush(hDC, brushOld);
+						DeleteBrush(brush);
+						SelectPen(hDC, penOld);
+						DeletePen(pen);
 						::ReleaseDC(hWnd, hDC);
 					}
 				}
